@@ -108,7 +108,9 @@ MVA.cv.quant <- function(X,Y,repet,k,ncomp,scale,model,lower,upper,Y.add,weights
 	train.set.Y <- as.matrix(as.data.frame(train.set[,col.Y]))
 	if (scale) {
 	  train.set.X <- scale(train.set.X)
+	  train.set.X <- verif.finite(train.set.X)
 	  test.set.X <- stand(test.set.X,train.set.X)
+	  test.set.X <- verif.finite(test.set.X)
 	}
 	nmax <- min(c(nrow(train.set)-max(unlist(lapply(test.sets.list.k,nrow))),ncol(X)+1))
 	if (ncomp>=nmax) {
@@ -168,7 +170,9 @@ MVA.cv.qual1 <- function(X,Y,Yfac,groups,repet,k,ncomp,scale,model,lower,upper,Y
 	train.set.trueclass <- trueclass[-as.numeric(rownames(test.set))]
 	if (scale) {
 	  train.set.X <- scale(train.set.X)
+	  train.set.X <- verif.finite(train.set.X)
 	  test.set.X <- stand(test.set.X,train.set.X)
+	  test.set.X <- verif.finite(test.set.X)
 	}
 	if (model %in% c("PLS-DA","PPLS-DA")) {
 	  nmax <- min(c(nrow(train.set),ncol(X)+1))
@@ -252,7 +256,9 @@ MVA.cv.qual2 <- function(X,Y,Yfac,groups,repet,k,ncomp,scale,model,lower,upper,Y
 	train.set.Yfac <- Yfac[-as.numeric(rownames(test.set))]
 	if (scale) {
 	  train.set.X <- scale(train.set.X)
+	  train.set.X <- verif.finite(train.set.X)
 	  test.set.X <- stand(test.set.X,train.set.X)
+	  test.set.X <- verif.finite(test.set.X)
 	}
 	nmax <- min(c(nrow(train.set),ncol(X)+1))
 	if (ncomp>=nmax) {
