@@ -1,5 +1,5 @@
 perm.anova <-
-function(formula,nest.f2=c("fixed","random"),data,nperm=999) {
+function(formula,nest.f2=c("fixed","random"),data,nperm=999,progress=TRUE) {
   if (missing(formula)||(length(formula)!=3)) {stop("missing or incorrect formula")}
   allnames <- all.names(formula)
   if (length(formula[[3]])==3) {
@@ -9,7 +9,7 @@ function(formula,nest.f2=c("fixed","random"),data,nperm=999) {
   m$formula <- formula
   if (is.matrix(eval(m$data,parent.frame()))) {m$data <- as.data.frame(m$data)}
   m[[1]] <- as.name("model.frame")
-  m$nest.f2 <- m$nperm <- NULL
+  m$nest.f2 <- m$nperm <- m$progress <- NULL
   mf <- eval(m,parent.frame())
   resp <- mf[,1]
   fact1 <- mf[,2]
@@ -18,18 +18,18 @@ function(formula,nest.f2=c("fixed","random"),data,nperm=999) {
   data.name <- variables[1]
   tab <- NULL
   if (length(variables)==2) {
-    tab <- .perm.anova.1way(resp,fact1,variables,nperm)$tab
+    tab <- .perm.anova.1way(resp,fact1,variables,nperm,progress)$tab
   } else if (length(variables)==3) {
     fact2 <- mf[,3]
     if (allnames[3]=="+") {
-	tab <- .perm.anova.2wayA(resp,fact1,fact2,variables,nperm)$tab
+	tab <- .perm.anova.2wayA(resp,fact1,fact2,variables,nperm,progress)$tab
     } else if (allnames[3]=="*") {
-	tab <- .perm.anova.2wayB(resp,fact1,fact2,variables,nperm)$tab
+	tab <- .perm.anova.2wayB(resp,fact1,fact2,variables,nperm,progress)$tab
     } else if (allnames[3]=="/") {
-	tab <- .perm.anova.2wayC(resp,fact1,fact2,nest.f2,variables,nperm)$tab
+	tab <- .perm.anova.2wayC(resp,fact1,fact2,nest.f2,variables,nperm,progress)$tab
     } else if (allnames[3]=="|") {
 	data.name <- paste(data.name,"\nBlock: ",variables[3],sep="")
-	tab <- .perm.anova.2wayD(resp,fact1,fact2,variables,nperm)$tab
+	tab <- .perm.anova.2wayD(resp,fact1,fact2,variables,nperm,progress)$tab
     }
   } else if (length(variables)==4) {
     if (allnames[3]!="|") {stop("incorrect 'formula'")}
@@ -37,9 +37,9 @@ function(formula,nest.f2=c("fixed","random"),data,nperm=999) {
     fact3 <- mf[,4]
     data.name <- paste(data.name,"\nBlock: ",variables[4],sep="")
     if (allnames[4]=="+") {
-	tab <- .perm.anova.3wayA(resp,fact1,fact2,fact3,variables,nperm)$tab
+	tab <- .perm.anova.3wayA(resp,fact1,fact2,fact3,variables,nperm,progress)$tab
     } else if (allnames[4]=="*") {
-	tab <- .perm.anova.3wayB(resp,fact1,fact2,fact3,variables,nperm)$tab
+	tab <- .perm.anova.3wayB(resp,fact1,fact2,fact3,variables,nperm,progress)$tab
     } else {
 	stop("only additive and multiplicative models are permitted")
     }
@@ -49,7 +49,7 @@ function(formula,nest.f2=c("fixed","random"),data,nperm=999) {
 }
 
 .perm.anova.1way <-
-function(resp,fact1,variables,nperm) {
+function(resp,fact1,variables,nperm,progress) {
   anova.ref <- anova(lm(resp~fact1))
   F.ref <- anova.ref[1,"F value"]
   tab <- data.frame("Sum Sq"=anova.ref[,"Sum Sq"],"Df"=anova.ref[,"Df"],"Mean Sq"=anova.ref[,"Mean Sq"],
@@ -57,11 +57,11 @@ function(resp,fact1,variables,nperm) {
   rownames(tab) <- c(variables[2],"Residuals")
   F.perm <- numeric(nperm+1)
   F.perm[1] <- anova.ref[1,"F value"]
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm <- anova(lm(sample(resp)~fact1))
     F.perm[i+1] <- anova.perm[1,"F value"]
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue <- length(which((F.perm+.Machine$double.eps/2) >= F.ref))/(nperm+1)
@@ -70,7 +70,7 @@ function(resp,fact1,variables,nperm) {
 }
 
 .perm.anova.2wayA <-
-function(resp,fact1,fact2,variables,nperm) {
+function(resp,fact1,fact2,variables,nperm,progress) {
   if (any(diff(tapply(resp,list(fact2,fact1),length))!=0)) {stop("this function is not made for unbalanced design")}
   anova.ref <- anova(lm(resp~fact1+fact2))
   F1.ref <- anova.ref[1,"F value"]
@@ -82,12 +82,12 @@ function(resp,fact1,fact2,variables,nperm) {
   F2.perm <- numeric(nperm+1)
   F1.perm[1] <- F1.ref
   F2.perm[1] <- F2.ref
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm <- anova(lm(sample(resp)~fact1+fact2))
     F1.perm[i+1] <- anova.perm[1,"F value"]
     F2.perm[i+1] <- anova.perm[2,"F value"]
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue1 <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
@@ -97,11 +97,11 @@ function(resp,fact1,fact2,variables,nperm) {
 }
 
 .perm.anova.2wayB <-
-function(resp,fact1,fact2,variables,nperm) {
+function(resp,fact1,fact2,variables,nperm,progress) {
   if (any(diff(tapply(resp,list(fact2,fact1),length))!=0)) {stop("this function is not made for unbalanced design")}
   if (tapply(resp,list(fact2,fact1),length)[1,1]==1) {
     warning("no repetition of ",variables[2],":",variables[3],", ANOVA without interaction")
-    tab <- .perm.anova.2wayA(resp,fact1,fact2,variables,nperm)$tab
+    tab <- .perm.anova.2wayA(resp,fact1,fact2,variables,nperm,progress)$tab
   } else {
     anova.ref <- anova(lm(resp~fact1*fact2))
     F1.ref <- anova.ref[1,"F value"]
@@ -116,13 +116,13 @@ function(resp,fact1,fact2,variables,nperm) {
     F1.perm[1] <- F1.ref
     F2.perm[1] <- F2.ref
     F3.perm[1] <- F3.ref
-    pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+    if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
     for (i in 1:nperm) {
 	anova.perm <- anova(lm(sample(resp)~fact1*fact2))
 	F1.perm[i+1] <- anova.perm[1,"F value"]
 	F2.perm[i+1] <- anova.perm[2,"F value"]
 	F3.perm[i+1] <- anova.perm[3,"F value"]
-	setTxtProgressBar(pb,round(i*100/nperm,0))
+	if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
     }
     cat("\n")
     pvalue1 <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
@@ -134,7 +134,7 @@ function(resp,fact1,fact2,variables,nperm) {
 }
 
 .perm.anova.2wayC <-
-function(resp,fact1,fact2,nest.f2,variables,nperm) {
+function(resp,fact1,fact2,nest.f2,variables,nperm,progress) {
   if (any(diff(tapply(resp,fact2,length))!=0)) {stop("this function is not made for unbalanced design")}
   anova.ref <- anova(lm(resp~fact1/fact2))
   if (nest.f2=="random") {anova.ref[1,"F value"] <- anova.ref[1,"Mean Sq"]/anova.ref[2,"Mean Sq"]}
@@ -147,7 +147,7 @@ function(resp,fact1,fact2,nest.f2,variables,nperm) {
   F2.perm <- numeric(nperm+1)
   F1.perm[1] <- F1.ref
   F2.perm[1] <- F2.ref
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm1 <- anova(lm(sample(resp)~fact1/fact2))
     if (nest.f2=="random") {
@@ -159,7 +159,7 @@ function(resp,fact1,fact2,nest.f2,variables,nperm) {
     ordre.new <- unlist(tapply(ordre,factor(rep(1:nlevels(fact1),each=length(fact1)/nlevels(fact1))),sample))
     anova.perm2 <- anova(lm(resp[ordre.new]~fact1/fact2))
     F2.perm[i+1] <- anova.perm2[2,"F value"]
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue1 <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
@@ -174,7 +174,7 @@ function(resp,fact1,fact2,nest.f2,variables,nperm) {
 }
 
 .perm.anova.2wayD <-
-function(resp,fact1,fact2,variables,nperm) {
+function(resp,fact1,fact2,variables,nperm,progress) {
   if (any(diff(tapply(resp,fact2,length))!=0)) {stop("this function is not made for unbalanced design")}
   if (tapply(resp,list(fact2,fact1),length)[1,1]==1) {stop("no repetition of ",variables[2],":",variables[3]," -> no interaction")}
   anova.ref <- anova(lm(resp~fact1*fact2))
@@ -185,11 +185,11 @@ function(resp,fact1,fact2,variables,nperm) {
   rownames(tab) <- c(variables[2],"Residuals")
   F1.perm <- numeric(nperm+1)
   F1.perm[1] <- F1.ref
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm <- anova(lm(sample(resp)~fact1*fact2))
     F1.perm[i+1] <- anova.perm[1,"Mean Sq"]/anova.perm[3,"Mean Sq"]
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
@@ -198,7 +198,7 @@ function(resp,fact1,fact2,variables,nperm) {
 }
 
 .perm.anova.3wayA <-
-function(resp,fact1,fact2,fact3,variables,nperm) {
+function(resp,fact1,fact2,fact3,variables,nperm,progress) {
   if (any(diff(table(fact1,fact2,fact3))!=0)) {stop("this function is not made for unbalanced design")}
   anova.ref <- anova(lm(resp~fact1*fact3+fact2*fact3))
   MSres <- sum(anova.ref[4:5,"Sum Sq"])/sum(anova.ref[4:5,"Df"]) 
@@ -211,13 +211,13 @@ function(resp,fact1,fact2,fact3,variables,nperm) {
   F2.perm <- numeric(nperm+1)
   F1.perm[1] <- F1.ref
   F2.perm[1] <- F2.ref
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm <- anova(lm(sample(resp)~fact1*fact3+fact2*fact3))
     MSres.perm <- sum(anova.perm[4:5,"Sum Sq"])/sum(anova.perm[4:5,"Df"]) 
     F1.perm[i+1] <- anova.perm[1,"Mean Sq"]/MSres.perm
     F2.perm[i+1] <- anova.perm[3,"Mean Sq"]/MSres.perm
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue1 <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
@@ -227,7 +227,7 @@ function(resp,fact1,fact2,fact3,variables,nperm) {
 }
 
 .perm.anova.3wayB <-
-function(resp,fact1,fact2,fact3,variables,nperm) {
+function(resp,fact1,fact2,fact3,variables,nperm,progress) {
   if (any(diff(table(fact1,fact2,fact3))!=0)) {stop("this function is not made for unbalanced design")}
   if (table(fact1,fact2,fact3)[1,1,1]==1) {stop("no repetition of ",variables[2],":",variables[3],":",variables[4]," -> no interaction")}
   anova.ref <- anova(lm(resp~fact1*fact2*fact3))
@@ -244,14 +244,14 @@ function(resp,fact1,fact2,fact3,variables,nperm) {
   F1.perm[1] <- F1.ref
   F2.perm[1] <- F2.ref
   F1F2.perm[1] <- F1F2.ref
-  pb <- txtProgressBar(min=0,max=100,initial=0,style=3)
+  if (progress) {pb <- txtProgressBar(min=0,max=100,initial=0,style=3)}
   for (i in 1:nperm) {
     anova.perm <- anova(lm(sample(resp)~fact1*fact2*fact3))
     MSres.perm <- sum(anova.perm[5:7,"Sum Sq"])/sum(anova.perm[5:7,"Df"]) 
     F1.perm[i+1] <- anova.perm[1,"Mean Sq"]/MSres.perm
     F2.perm[i+1] <- anova.perm[2,"Mean Sq"]/MSres.perm
     F1F2.perm[i+1] <- anova.perm[4,"Mean Sq"]/MSres.perm
-    setTxtProgressBar(pb,round(i*100/nperm,0))
+    if (progress) {setTxtProgressBar(pb,round(i*100/nperm,0))}
   }
   cat("\n")
   pvalue1 <- length(which((F1.perm+.Machine$double.eps/2) >= F1.ref))/(nperm+1)
